@@ -1,11 +1,11 @@
-# $Id: SQLite.pm,v 1.47 2005/06/20 13:53:01 matt Exp $
+# $Id: SQLite.pm,v 1.49 2005/12/01 20:48:05 matt Exp $
 
 package DBD::SQLite;
 use strict;
 
 use DBI;
 use vars qw($err $errstr $state $drh $VERSION @ISA);
-$VERSION = '1.09';
+$VERSION = '1.10';
 
 use DynaLoader();
 @ISA = ('DynaLoader');
@@ -348,7 +348,30 @@ limited by the typeless nature of the SQLite database.
 
 =item sqlite_version
 
-Returns the version of the SQLite library which DBD::SQLite is using, e.g., "2.8.0".
+Returns the version of the SQLite library which DBD::SQLite is using,
+e.g., "2.8.0". Can only be read.
+
+=item unicode
+
+If set to a true value, DBD::SQLite will turn the UTF-8 flag on for all text
+strings coming out of the database. For more details on the UTF-8 flag see
+L<perlunicode>. The default is for the UTF-8 flag to be turned off.
+
+Also note that due to some bizareness in SQLite's type system (see
+http://www.sqlite.org/datatype3.html), if you want to retain
+blob-style behavior for B<some> columns under C<< $dbh->{unicode} = 1
+>> (say, to store images in the database), you have to state so
+explicitely using the 3-argument form of L<DBI/bind_param> when doing
+updates:
+
+    use DBI qw(:sql_types);
+    $dbh->{unicode} = 1;
+    my $sth = $dbh->prepare
+         ("INSERT INTO mytable (blobcolumn) VALUES (?)");
+    $sth->bind_param(1, $binary_data, SQL_BLOB); # binary_data will
+    # be stored as-is.
+
+Defining the column type as BLOB in the DDL is B<not> sufficient.
 
 =back
 
@@ -367,7 +390,7 @@ DBI.
 
 Retrieve the current busy timeout.
 
-=head2 $dbh->finc( $ms, 'busy_timeout' )
+=head2 $dbh->func( $ms, 'busy_timeout' )
 
 Set the current busy timeout. The timeout is in milliseconds.
 
@@ -494,6 +517,32 @@ The aggregate function can then be used as:
     SELECT group_name, variance(score) FROM results
     GROUP BY group_name;
 
+=head1 BLOBS
+
+SQLite fully supports blobs, but they're a little complex to store and retrieve
+from the database due to the typeless nature of SQLite. In order to make proper
+use of blobs you need to explicitly bind variables to a prepared statement using
+a BLOB type:
+
+  use DBI qw(:sql_types);
+  my $dbh = DBI->connect("dbi:sqlite:/path/to/db");
+  
+  my $blob = `cat foo.jpg`;
+  my $sth = $dbh->prepare("INSERT INTO mytable VALUES (1, ?)");
+  $sth->bind_param(1, $blob, SQL_BLOB);
+  $sth->execute();
+
+And then retreival just works:
+
+  $sth = $dbh->prepare("SELECT * FROM mytable WHERE id = 1");
+  $sth->execute();
+  my $row = $sth->fetch;
+  my $blobo = $row->[1];
+  
+  # now $blobo == $blob
+
+This is all necessary because SQLite is typeless. Sorry.
+
 =head1 NOTES
 
 To access the database from the command line, try using dbish which comes with
